#define MAX_THREADS 128
#define MAX_SUSPEND 1024
#define MAX_SNOOZE  8192
#define DEFAULT_TARGET_MIME ""

class Resyncer : public BApplication {
 public:
	Resyncer()
		: BApplication("application/x.vnd-PDW.Resyncer") {
		strcpy(targetMime, DEFAULT_TARGET_MIME);
		srand(real_time_clock_usecs());
		// want to interrupt target application at any point
		set_thread_priority(find_thread(0), B_REAL_TIME_PRIORITY);
	}
	// read MIME type from command line
	void ArgvReceived(int32 argc, char **argv) {
		if(argc>1) strcpy(targetMime, argv[1]);
	}
	// read MIME type from ref and start target application
	void RefsReceived(BMessage *msg) {
		app_info info;
		static entry_ref ref;
		uint32 type; int32 cnt;
		msg->GetInfo("refs", &type, &cnt); 
		if(type!=B_REF_TYPE || cnt==0) return; 
		if(msg->FindRef("refs", 0, &ref)==B_OK) {
			be_roster->Launch(&ref);
			be_roster->GetAppInfo(&ref, &info);
			strcpy(targetMime, info.signature);
		}
	}
	void ReadyToRun() {
		app_info ainfo;
		// loop as long as target app is running
		while(be_roster->GetAppInfo(targetMime, &ainfo)==B_OK) {
			team_info tinfo;
			get_team_info(ainfo.team, &tinfo);
	
			int tnr=0;
			int32 cookie=0;
			thread_info thinfo;
			thread_id threads[MAX_THREADS];
	
			// construct a list of target threads
			while(get_next_thread_info(ainfo.team, &cookie, &thinfo)==B_OK)
				threads[tnr++]=thinfo.thread;			
	
			// suspend a thread for some usecs and change its priority
			for(int i=0; i<64; i++) {
				int r=random();
				int tmp=r%tnr; r>>=7;
				suspend_thread(threads[tmp]);
				snooze(r%MAX_SUSPEND); r>>=10;
				resume_thread(threads[tmp]);
				snooze(r%MAX_SNOOZE); r>>=13;
				set_thread_priority(threads[tmp], random()%128+1);
			}
		}
		PostMessage(B_QUIT_REQUESTED);
	}
 private:
	char targetMime[B_MIME_TYPE_LENGTH];
};

int main(int argc, char *argv[]) {
	BApplication *app=new Resyncer();
	app->Run();
	delete app;
	return 0;
}
